# -*- coding: utf-8 -*-
import inspect
from functools import wraps
from string import Formatter

from pydantic.main import BaseModel

from easi_py_common.client.rest_client import RestClient, RestClientResponseDecoder

service_endpoint = {}

HTTP_METHOD_GET = "GET"
HTTP_METHOD_POST = "POST"
HTTP_METHOD_PUT = "PUT"
HTTP_METHOD_DELETE = "DELETE"


def service_client(
    url,
    timeout=(5, 5),
    http_adapter=None,
    before_request=None,
    response_decoder: RestClientResponseDecoder = None,
):
    def wrapper(f):
        key = "{}.{}".format(f.__module__, f.__name__)

        service_endpoint[key] = RestClient(
            url,
            timeout=timeout,
            http_adapter=http_adapter,
            before_request=before_request,
            response_decoder=response_decoder,
        )

        @wraps(f)
        def wrapped(*args, **kwargs):
            return f(*args, **kwargs)

        return wrapped

    return wrapper


def service_mapping(uri, method, timeout=None, data_by_alias: bool = True, data_key="data"):
    def wrapper(f):
        full_arg_spec = inspect.getfullargspec(f)
        is_method = full_arg_spec.args and full_arg_spec.args[0] == "self"
        if not is_method:
            raise Exception("{} is invalid service_mapping", f.__name__)

        dst = full_arg_spec.annotations.get("return")
        has_format_names = [fn for _, fn, _, _ in Formatter().parse(uri) if fn is not None]
        has_format_names.append("self")  # 参数列表过滤self

        def __get_request_params(named_args):
            request_named_args = {k: v for k, v in named_args.items() if k not in has_format_names}
            data = None
            if method == HTTP_METHOD_POST or method == HTTP_METHOD_PUT:
                if data_key in request_named_args:
                    data = request_named_args.pop(data_key)
                if data is None:
                    data = {}

            return request_named_args, data

        def __get_function_args(args, kwargs):
            named_args = inspect.getcallargs(f, *args, **kwargs)
            named_args.update(named_args.pop(full_arg_spec.varkw, {}))
            if (
                (method == HTTP_METHOD_GET or method == HTTP_METHOD_DELETE)
                and data_key in named_args
                and isinstance(named_args[data_key], BaseModel)
            ):
                params_data = named_args.pop(data_key)
                named_args.update(params_data.dict(by_alias=data_by_alias))
            return named_args

        @wraps(f)
        def wrapped(*args, **kwargs):
            name = "{}.{}".format(args[0].__class__.__module__, args[0].__class__.__name__)
            if name not in service_endpoint:
                raise Exception("{}.{} is not have config endpoint", f.__module__, f.__name__)

            named_args = __get_function_args(args, kwargs)
            _uri = uri.format(**named_args)
            params, data = __get_request_params(named_args)

            client = service_endpoint[name]
            if method == HTTP_METHOD_GET:
                return client.get(_uri, params=params, timeout=timeout, dst=dst)
            elif method == HTTP_METHOD_POST:
                return client.post(
                    _uri,
                    json_data=data,
                    params=params,
                    timeout=timeout,
                    dst=dst,
                    request_by_alias=data_by_alias,
                )
            elif method == HTTP_METHOD_PUT:
                return client.put(
                    _uri,
                    json_data=data,
                    params=params,
                    timeout=timeout,
                    dst=dst,
                    request_by_alias=data_by_alias,
                )
            elif method == HTTP_METHOD_DELETE:
                return client.delete(_uri, params=params, timeout=timeout, dst=dst)
            else:
                raise Exception("{}.{} is invalid method", f.__module__, f.__name__)

        return wrapped

    return wrapper


def service_mapping_get(uri, timeout=None, data_by_alias: bool = True, data_key="data"):
    return service_mapping(
        uri, method=HTTP_METHOD_GET, timeout=timeout, data_by_alias=data_by_alias, data_key=data_key
    )


def service_mapping_post(uri, timeout=None, data_by_alias: bool = True, data_key="data"):
    return service_mapping(
        uri,
        method=HTTP_METHOD_POST,
        timeout=timeout,
        data_by_alias=data_by_alias,
        data_key=data_key,
    )


def service_mapping_put(uri, timeout=None, request_by_alias: bool = True, data_key="data"):
    return service_mapping(
        uri,
        method=HTTP_METHOD_PUT,
        timeout=timeout,
        data_by_alias=request_by_alias,
        data_key=data_key,
    )


def service_mapping_delete(uri, timeout=None, request_by_alias: bool = True, data_key="data"):
    return service_mapping(
        uri,
        method=HTTP_METHOD_DELETE,
        timeout=timeout,
        data_by_alias=request_by_alias,
        data_key=data_key,
    )



import json
from typing import Optional

import boto3
import yaml


class ContentType:
    json = "json"
    yaml = "yaml"


class SecretsManager:
    def __init__(
        self,
        aws_access_key_id: str,
        aws_secret_access_key: str,
        region_name: str,
        aws_session_token: Optional[str] = None,
        **kwargs,
    ):
        session = boto3.session.Session(
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            region_name=region_name,
            aws_session_token=aws_session_token,
        )
        self.client = session.client(service_name="secretsmanager", **kwargs)

    def create_secrets(self, name: str, secret: str, **kwargs):
        response = self.client.create_secret(Name=name, SecretString=secret, **kwargs)
        return response

    def get_secret(
        self,
        secret_id: str,
        content_type: ContentType = ContentType.json,
        version_stage: Optional[str] = None,
        version_id: Optional[str] = None,
    ):
        kwargs = {"SecretId": secret_id}
        if version_stage is not None:
            kwargs["VersionStage"] = version_stage
        if version_id is not None:
            kwargs["VersionId"] = version_id
        response = self.client.get_secret_value(**kwargs)
        secret = response["SecretString"]
        if content_type == ContentType.json:
            return json.loads(secret)
        elif content_type == ContentType.yaml:
            return yaml.load(secret)
        else:
            raise ValueError("Unknown content type")



# -*- coding: utf-8 -*-
import json
import time
import traceback
from abc import abstractmethod
from datetime import datetime
from typing import Dict, Optional

import pytz
import requests
from flask import current_app
from pydantic import Field
from pydantic.main import BaseModel, ModelMetaclass
from requests import Response
from requests.adapters import HTTPAdapter

from easi_py_common.core.error import RestClientException, ServiceException

HEADER = {"Content-Type": "application/json; charset=UTF-8"}
DST_MODE_SERVICE = "service"
DST_MODE_OTHER = "other"


class ClientRequestData(BaseModel):
    url: str = Field(..., description="url")
    method: str = Field(..., description="method")
    json_data: Optional[Dict] = Field(description="url")
    params: Optional[Dict] = Field(description="url")
    timeout: object = Field(..., description="url")
    other_kwargs: Optional[Dict] = Field(description="url")
    dst: Optional[object] = Field(description="url")
    request_time: int
    request_time_str: str
    response_time: int
    response_time_str: str

    class Config:
        arbitrary_types_allowed = True


class RestClientResponseDecoder:
    @abstractmethod
    def get_data(self, request_data: ClientRequestData, response: Response):
        pass


class ServiceRestClientResponseDecoder(RestClientResponseDecoder):
    def __init__(self, dst_mode=DST_MODE_SERVICE):
        self.dst_mode = dst_mode

    def get_data(self, request_data: ClientRequestData, response: Response):
        dst = request_data.dst
        status_code = response.status_code
        if status_code == 200:
            result = response.json()
            if (
                isinstance(result, bool)
                or isinstance(result, int)
                or isinstance(result, float)
                or isinstance(result, str)
            ):
                return result
            if self.dst_mode == DST_MODE_SERVICE:
                code = result.get("code", 0)
                if code != 0:
                    raise ServiceException(code=code, msg=result.get("message", ""))
                result = result.get("data")

            if dst and dst.__class__ == ModelMetaclass:
                return dst.parse_obj(result)
            return result

        if 400 <= status_code < 500:
            raise RestClientException(code=status_code, msg="Internal Server Error")
        else:
            raise RestClientException(code=status_code, msg="Internal Server Error")


def _now_log_datetime() -> (int, str):
    return (
        int(time.time() * 1000),
        datetime.strftime(
            datetime.now(pytz.timezone("Etc/GMT-8")).replace(tzinfo=None), "%Y.%m.%d %H:%M:%S"
        ),
    )


class LogServiceRestClientResponseDecoder(ServiceRestClientResponseDecoder):
    def __init__(self, dst_mode=DST_MODE_SERVICE):
        super().__init__(dst_mode)
        self.dst_mode = dst_mode

    @abstractmethod
    def success_log(self, request_data: ClientRequestData, response: Response, response_data):
        pass

    @abstractmethod
    def error_log(self, request_data: ClientRequestData, response: Response, exception: Exception):
        pass

    def get_data(self, request_data: ClientRequestData, response: Response):
        try:
            response_data = super().get_data(request_data, response)

            try:
                self.success_log(request_data, response, response_data)
            except Exception as success_log_e:
                current_app.logger.error(
                    "success_log: {}\n{}".format(success_log_e, traceback.format_exc())
                )

            return response_data
        except Exception as e:
            try:
                self.error_log(request_data, response, e)
            except Exception as error_log_e:
                current_app.logger.error(
                    "error_log: {}\n{}".format(error_log_e, traceback.format_exc())
                )

            raise e


class RestClient:
    def __init__(
        self,
        url,
        timeout=(5, 5),
        http_adapter=None,
        before_request=None,
        response_decoder: RestClientResponseDecoder = None,
    ):
        self.url = url
        if http_adapter is None:
            http_adapter = HTTPAdapter(pool_connections=10, pool_maxsize=10, max_retries=0)
        session = requests.Session()
        session.mount("http://", http_adapter)
        session.mount("https://", http_adapter)

        self.session = session
        self.timeout = timeout
        self.before_request = before_request
        self.response_decoder = response_decoder
        if not self.response_decoder:
            self.response_decoder = ServiceRestClientResponseDecoder(dst_mode=DST_MODE_SERVICE)

    def _get_url(self, uri):
        if "SERVICE_CLIENT" in current_app.config and "http" not in self.url:
            service_client_keys = current_app.config["SERVICE_CLIENT"]
            self.url = self.url.format(**service_client_keys)
        return self.url + uri

    def get(self, uri, params=None, timeout=None, dst=None, **kwargs):
        if timeout is None:
            timeout = self.timeout
        request_time, request_time_str = _now_log_datetime()
        _url = self._get_url(uri)

        if callable(self.before_request):
            _request_parms = {
                "url": _url,
                "method": "GET",
                "params": params,
            }
            _kwargs = self.before_request(_request_parms)
            kwargs.update(_kwargs)

        resp = self.session.get(_url, params=params, timeout=timeout, **kwargs)
        response_time, response_time_str = _now_log_datetime()
        request_data = ClientRequestData(
            url=_url,
            method="GET",
            json_data=None,
            params=params,
            timeout=timeout,
            other_kwargs=kwargs,
            dst=dst,
            request_time=request_time,
            request_time_str=request_time_str,
            response_time=response_time,
            response_time_str=response_time_str,
        )
        return self._do_response(resp, request_data)

    def post(
        self,
        uri,
        json_data=None,
        params=None,
        timeout=None,
        dst=None,
        request_by_alias: bool = True,
        **kwargs,
    ):
        if timeout is None:
            timeout = self.timeout
        request_time, request_time_str = _now_log_datetime()
        _url = self._get_url(uri)
        data = None
        if json_data:
            data = (
                json.dumps(json_data.dict(by_alias=request_by_alias))
                if isinstance(json_data, BaseModel)
                else json.dumps(json_data)
            )

        if callable(self.before_request):
            _request_parms = {
                "url": _url,
                "method": "POST",
                "data": data,
                "params": params,
            }
            _kwargs = self.before_request(_request_parms)
            kwargs.update(_kwargs)

        if "headers" in kwargs:
            kwargs.get("headers").update(HEADER)
        else:
            kwargs["headers"] = HEADER

        resp = self.session.post(_url, data=data, params=params, timeout=timeout, **kwargs)
        response_time, response_time_str = _now_log_datetime()
        request_data = ClientRequestData(
            url=_url,
            method="POST",
            json_data=json_data,
            params=params,
            timeout=timeout,
            other_kwargs=kwargs,
            dst=dst,
            request_time=request_time,
            request_time_str=request_time_str,
            response_time=response_time,
            response_time_str=response_time_str,
        )
        return self._do_response(resp, request_data)

    def put(
        self,
        uri,
        json_data=None,
        params=None,
        timeout=None,
        dst=None,
        request_by_alias: bool = True,
        **kwargs,
    ):
        if timeout is None:
            timeout = self.timeout
        request_time, request_time_str = _now_log_datetime()
        _url = self._get_url(uri)

        data = None
        if json_data:
            data = (
                json.dumps(json_data.dict(by_alias=request_by_alias))
                if isinstance(json_data, BaseModel)
                else json.dumps(json_data)
            )

        if callable(self.before_request):
            _request_parms = {
                "url": _url,
                "method": "PUT",
                "data": data,
                "params": params,
            }
            _kwargs = self.before_request(_request_parms)
            kwargs.update(_kwargs)

        if "headers" in kwargs:
            kwargs.get("headers").update(HEADER)
        else:
            kwargs["headers"] = HEADER

        resp = self.session.put(_url, data=data, params=params, timeout=timeout, **kwargs)
        response_time, response_time_str = _now_log_datetime()
        request_data = ClientRequestData(
            url=_url,
            method="PUT",
            json_data=json_data,
            params=params,
            timeout=timeout,
            other_kwargs=kwargs,
            dst=dst,
            request_time=request_time,
            request_time_str=request_time_str,
            response_time=response_time,
            response_time_str=response_time_str,
        )
        return self._do_response(resp, request_data)

    def delete(self, uri, params=None, timeout=None, dst=None, **kwargs):
        if timeout is None:
            timeout = self.timeout

        request_time, request_time_str = _now_log_datetime()

        _url = self._get_url(uri)
        if callable(self.before_request):
            _request_parms = {
                "url": _url,
                "method": "DELETE",
                "params": "params",
            }
            _kwargs = self.before_request(_request_parms)
            kwargs.update(_kwargs)

        resp = self.session.delete(_url, params=params, timeout=timeout, **kwargs)

        response_time, response_time_str = _now_log_datetime()
        request_data = ClientRequestData(
            url=_url,
            method="DELETE",
            json_data=None,
            params=params,
            timeout=timeout,
            other_kwargs=kwargs,
            dst=dst,
            request_time=request_time,
            request_time_str=request_time_str,
            response_time=response_time,
            response_time_str=response_time_str,
        )
        return self._do_response(resp, request_data)

    def _do_response(self, resp, request_data: ClientRequestData):

        return self.response_decoder.get_data(request_data, resp)



# -*- coding: utf-8 -*-
from typing import Tuple, List


def map_group(data, key=None):
    m = {}
    for val in data:
        ret = key(val)
        k = ret
        if isinstance(ret, Tuple) or isinstance(ret, List):
            if len(ret) > 1:
                k = ret[0]
                val = ret[1]
        if k not in m:
            m[k] = []
        m[k].append(val)
    return m



import json
import time
import uuid

import boto3
import orjson
from botocore.config import Config

DEAD_QUEUE_NAME = "easipay_letter_dead"
DEAD_QUEUE_NAME_FIFO = "easipay_letter_dead.fifo"


def get_utc_timestamp():
    timestamp = time.time()
    return int(timestamp * 1000)


class SQSClient:
    __queue_map = {}
    __resource = None
    __dead_queue = None
    __dead_queue_fifo = None

    def __init__(
        self,
        region_name,
        queue_name_prefix="",
        aws_access_key_id="",
        aws_secret_access_key="",
        max_pool_connections=50,
        max_attempts=3,
    ):
        self.queue_name_prefix = queue_name_prefix
        config = Config(
            region_name=region_name,
            signature_version="v4",
            max_pool_connections=max_pool_connections,
            retries={
                "max_attempts": max_attempts,
                # 'mode': 'standard'
            },
        )

        self.__client = boto3.client(
            service_name="sqs",
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            config=config,
        )
        self.__init_queue()

    def send_message(self, queue_name, body, key="", group_id="", delay_seconds=0, cb=None):
        if ".fifo" in queue_name:
            if not group_id:
                raise Exception("MessageGroupId is required")

        return self.__send_message(
            queue_name, body, key, group_id, delay_seconds, cb, get_utc_timestamp()
        )

    def set_queue_attributes(self, queue_url, attributes):
        self.__client.set_queue_attributes(QueueUrl=queue_url, Attributes=attributes)

    def get_queue_url(self, queue_name):
        url = self.__get_url_by_name(queue_name)
        return url

    def get_queue_attr(self, qurl, attr_names):
        return self.__client.get_queue_attributes(QueueUrl=qurl, AttributeNames=attr_names)

    def get_sqs_client(self):
        return self.__client

    def __init_queue(self):
        if len(self.queue_name_prefix) > 0:
            queue_data = self.__client.list_queues(QueueNamePrefix=self.queue_name_prefix)
        else:
            queue_data = self.__client.list_queues()

        if "QueueUrls" in queue_data:
            for queue_url in queue_data["QueueUrls"]:
                name = self.__get_name_by_url(queue_url)
                self.__queue_map[name] = queue_url

        if not self.__dead_queue:
            self.__install_dead(DEAD_QUEUE_NAME)
            self.__install_dead(DEAD_QUEUE_NAME_FIFO)

    def __get_name_by_url(self, q_url):
        return q_url[q_url.rindex("/") + 1 :]

    def __get_url_by_name(self, queue_name):
        if queue_name not in self.__queue_map:
            qurl = self.__get_queue_url(queue_name)
            self.__queue_map[queue_name] = qurl
        return self.__queue_map[queue_name]

    def __send_message(
        self,
        queue_name,
        body,
        key="",
        group_id="",
        delay_seconds=0,
        cb=None,
        send_start_time=get_utc_timestamp(),
    ):
        execute_start_time = get_utc_timestamp()
        work_wait_time = int(execute_start_time - send_start_time)
        msg_body = dict(t=execute_start_time, msg=body, key=key)
        try:
            queue_url = self.__get_queue_url(queue_name)

            if ".fifo" in queue_name:
                response = self.__client.send_message(
                    QueueUrl=queue_url,
                    MessageBody=json.dumps(msg_body),
                    MessageGroupId=str(group_id),
                    MessageDeduplicationId=str(uuid.uuid1()),
                )
            else:
                if delay_seconds > 0:
                    if delay_seconds > 7200:
                        delay_seconds = 7200
                    msg_body["delay_seconds"] = delay_seconds

                if delay_seconds > 900:
                    delay_seconds = 900

                response = self.__client.send_message(
                    QueueUrl=queue_url,
                    DelaySeconds=delay_seconds,
                    MessageBody=json.dumps(msg_body),
                )

            msg_id = ""
            if "MessageId" in response:
                msg_id = response["MessageId"]

            if "Code" in response:
                msg_id = response["Code"]

            if cb:
                log_data = dict(
                    msg=msg_body["msg"],
                    t=msg_body["t"],
                    s_t=get_utc_timestamp(),
                    key=key,
                    group_id=group_id,
                    url=queue_url,
                    address=queue_name,
                    delay_seconds=msg_body["delay_seconds"] if "delay_seconds" in msg_body else 0,
                    work_wait_time=work_wait_time,
                )
                cb(msg_id, log_data)
                # self.__pool_work_log.submit(cb, msg_id, log_data)
            return True
        except Exception as e:
            if cb:
                log_data = dict(
                    msg=msg_body["msg"],
                    t=msg_body["t"],
                    s_t=get_utc_timestamp(),
                    key=key,
                    group_id=group_id,
                    url=e.message[0:254],
                    address=queue_name,
                    delay_seconds=msg_body["delay_seconds"] if "delay_seconds" in msg_body else 0,
                    work_wait_time=work_wait_time,
                )

                cb("fail", log_data)
        return False

    def send_message_batch(self, queue_name, msgs, cb=None):
        if queue_name not in self.__queue_map:
            qurl = self.__get_queue_url(queue_name)
            self.__queue_map[queue_name] = qurl

        queue_url = self.__queue_map[queue_name]
        is_fifo = False
        if ".fifo" in queue_name:
            is_fifo = True

        entries = []
        for msg in msgs:
            if is_fifo:
                entries.append(
                    {
                        "MessageBody": orjson.dumps(msg["body"]),
                        "MessageGroupId": str(msg["group_id"]),
                        "MessageDeduplicationId": str(uuid.uuid1()),
                    }
                )
            else:
                delay_seconds = msg.get("delay_seconds", 0)
                if delay_seconds < 0:
                    delay_seconds = 0

                if delay_seconds > 0:
                    if delay_seconds > 7200:
                        delay_seconds = 7200

                if delay_seconds > 900:
                    delay_seconds = 900

                entries.append(
                    {"MessageBody": orjson.dumps(msg["body"]), "DelaySeconds": delay_seconds}
                )
        response = None
        try:
            response = self.__client.send_message_batch(QueueUrl=queue_url, Entries=entries)
        except Exception as e:
            if cb:
                cb("fail", {"response": response, "queue_url": queue_url, "entries": entries})

    def __get_queue_url(
        self,
        queue_name,
        MessageRetentionPeriod=86400 * 3,
        receive_message_wait_time_seconds=10,
        visibility_timeout=300,
        max_receive_count=6,
    ):
        if queue_name in self.__queue_map:
            return self.__queue_map[queue_name]

        try:
            queue = self.__client.get_queue_url(
                QueueName=queue_name,
            )
            if "QueueUrl" in queue:
                return queue["QueueUrl"]
        except:
            pass

        attributes = {
            "VisibilityTimeout": str(visibility_timeout),
            "DelaySeconds": "0",
            "MessageRetentionPeriod": str(MessageRetentionPeriod),
            "ReceiveMessageWaitTimeSeconds": str(receive_message_wait_time_seconds),
        }

        if ".fifo" in queue_name:
            attributes["FifoQueue"] = "true"
            if self.__dead_queue_fifo:
                attributes["RedrivePolicy"] = json.dumps(
                    {
                        "deadLetterTargetArn": self.__dead_queue_fifo["arn"],
                        "maxReceiveCount": str(max_receive_count),
                    }
                )
        else:
            if self.__dead_queue:
                attributes["RedrivePolicy"] = json.dumps(
                    {
                        "deadLetterTargetArn": self.__dead_queue["arn"],
                        "maxReceiveCount": str(max_receive_count),
                    }
                )

        queue = self.__client.create_queue(QueueName=queue_name, Attributes=attributes)
        queue_url = queue["QueueUrl"]
        return queue_url

    def __install_dead(self, dead_name):
        dead_queue_url = self.__get_queue_url(dead_name, MessageRetentionPeriod=86400 * 10)

        response = self.__client.get_queue_attributes(
            QueueUrl=dead_queue_url, AttributeNames=["QueueArn"]
        )
        if ".fifo" in dead_name:
            self.__dead_queue_fifo = {
                "url": dead_queue_url,
                "arn": response["Attributes"]["QueueArn"],
            }
        else:
            self.__dead_queue = {"url": dead_queue_url, "arn": response["Attributes"]["QueueArn"]}



import uuid

from flask import current_app
from itsdangerous import SignatureExpired
from itsdangerous import TimedJSONWebSignatureSerializer as Serializer

from easi_py_common.core.error import InvalidTokenException, TokenExpiredException
from easi_py_common.jwt.constants import JwtConstants


class Jwt:
    def __init__(self, secret_key: str = "", refresh_secret_key: str = ""):
        if not secret_key:
            secret_key = current_app.config["JWT_SECRET_KEY"]
            refresh_secret_key = current_app.config["JWT_REFRESH_SECRET_KEY"]
        self.secret_key = secret_key
        self.refresh_secret_key = refresh_secret_key

    def create_token(
        self, id: int, expires_in: int = JwtConstants.DEFAULT_EXPIRED_IN, city_id: int = None
    ) -> str:
        s = self.__get_jw_serializer(self.secret_key, expires_in)
        return self.__create_token(s, id, city_id)

    def create_refresh_token(
        self, id: int, expires_in: int = JwtConstants.DEFAULT_REFRESH_EXPIRED_IN
    ) -> str:
        s = self.__get_jw_serializer(self.refresh_secret_key, expires_in)
        return self.__create_token(s, id)

    def parse_token(self, token: str) -> int:
        s = self.__get_jw_serializer(self.secret_key)
        return self.__parse_token(s, token)

    def parse_refresh_token(self, refresh_token: str) -> int:
        s = self.__get_jw_serializer(self.refresh_secret_key)
        return self.__parse_token(s, refresh_token)

    def __create_token(self, s: Serializer, id: int, city_id=None):
        if city_id:
            return s.dumps({"id": id, "uid": uuid.uuid1().__str__(), "city_id": city_id}).decode(
                "ascii"
            )
        else:
            return s.dumps({"id": id, "uid": uuid.uuid1().__str__()}).decode("ascii")

    def __parse_token(self, s: Serializer, token: str) -> int:
        try:
            data = s.loads(token)
        except SignatureExpired:
            raise TokenExpiredException()
        except Exception:
            raise InvalidTokenException()
        if "id" not in data:
            raise InvalidTokenException()
        return int(data["id"])

    def __parse_city_id(self, s: Serializer, token: str) -> int:
        try:
            data = s.loads(token)
        except SignatureExpired:
            raise TokenExpiredException()
        except Exception:
            raise InvalidTokenException()
        if "city_id" not in data:
            raise InvalidTokenException()
        return int(data["city_id"])

    @staticmethod
    def __parse_token_expire_timestamp(s: Serializer, token: str) -> int:
        try:
            header = s.loads(token, return_header=True)[1]
        except SignatureExpired:
            raise TokenExpiredException()
        except Exception:
            raise InvalidTokenException()

        return int(header["exp"])

    def token_expire_timestamp(self, token: str) -> int:
        s = self.__get_jw_serializer(self.secret_key)
        return self.__parse_token_expire_timestamp(s, token)

    def token_city_id(self, token: str) -> int:
        s = self.__get_jw_serializer(self.secret_key)
        return self.__parse_city_id(s, token)

    def __get_jw_serializer(self, secret_key: str, expires_in: int = 3600) -> Serializer:
        return Serializer(
            secret_key, expires_in=expires_in, algorithm_name=JwtConstants.SERIALIZER_ALGORITHM_NAME
        )


# if __name__ == '__main__':
#     from easi_py_common.core.flasky import AppWrapper
#     app_wrapper = AppWrapper('api')
#     app = app_wrapper.get_app()
#     app.app_context().push()
#
#     token = Jwt().create_token(1)
#     print(token)
#     city_id = Jwt().token_city_id(token)
#     print(city_id)



# -*- coding: utf-8 -*-


class ConsumerLogConstants:
    PUB_LOG = "{}_pub_log"
    SUB_LOG = "{}_sub_log"

    CONSUMER_LOG_ES_BULK_NUM = 1000
    CONSUMER_LOG_ES_BULK_THREAD_NUM = 8
    CONSUMER_LOG_ES_BULK_BUFFER_SIZE = (
        CONSUMER_LOG_ES_BULK_NUM * CONSUMER_LOG_ES_BULK_THREAD_NUM * 2
    )
    CONSUMER_LOG_MAX_SLEEP_ES_BULK_BUFFER_SIZE = CONSUMER_LOG_ES_BULK_BUFFER_SIZE * 1.5


# 日志
PUB_LOG = "pub_log"
SUB_LOG = "sub_log"
PUB_LOG_BULK_SAVE = "pub_log_bulk_save"
SUB_LOG_BULK_SAVE = "sub_log_bulk_save"



from easi_py_common.id.id_generator import RedisIdGenerator


class IdGenerator:
    @classmethod
    def install(cls, provider: str):
        """
        安装IdGenerator
        :param provider: 当前可取值redis
        :return:
        """
        assert provider in ["redis"]
        if provider == "redis":
            cls.instance = RedisIdGenerator()

    @classmethod
    def next_id(cls, *args, **kwargs):
        return cls.instance.next_id(*args, **kwargs)



from easi_py_common.core.redis import redis_store


class RedisIdGenerator:
    lua_script = """
    -- need redis 3.2+
    redis.replicate_commands();

    local prefix = '__idgenerator_';
    local partitionCount = 4096;
    local step = 3;
    local startStep = 0;

    local tag = KEYS[1];
    -- if user do not pass shardId, default partition is 0.
    local partition
    if KEYS[2] == nil then
      partition = 0;
    else
      partition = KEYS[2] % partitionCount;
    end

    local now = redis.call('TIME');

    local miliSecondKey = prefix .. tag ..'_' .. partition .. '_' .. now[1] .. '_' .. math.floor(now[2]/1000);

    local count;
    repeat
      count = tonumber(redis.call('INCRBY', miliSecondKey, step));
      if count > (1024 - step) then
          now = redis.call('TIME');
          miliSecondKey = prefix .. tag ..'_' .. partition .. '_' .. now[1] .. '_' .. math.floor(now[2]/1000);
      end
    until count <= (1024 - step)

    if count == step then
      redis.call('PEXPIRE', miliSecondKey, 5);
    end

    -- second, microSecond, partition, seq
    return {tonumber(now[1]), tonumber(now[2]), partition, count + startStep}
    """
    epoch = 1620921600000

    def __init__(self):
        self.cmd = redis_store.register_script(self.lua_script)

    def next_id(self, service_name: str) -> int:
        """
        生成分布式唯一ID
        :param service_name: 服务名称，可随意定义
        :return:
        """
        ret = self.cmd(keys=["service_name"], args=[service_name])
        second = ret[0]
        micro_second = ret[1]
        seq = ret[3]
        milli_second = second * 1000 + int(micro_second / 1000)
        return (milli_second - self.epoch) << (12 + 10) + seq



# -*- coding: utf-8 -*-
import json
import time
import traceback
from datetime import datetime
from inspect import isfunction

from flask import current_app

from easi_py_common.core.redis import RedisLock, redis_store
from easi_py_common.sqs import utils
from easi_py_common.sqs.constants import ConsumerLogConstants


def get_consumer_log_es_bulk_key():
    # 生成锁key
    consumer_log_es_bulk_key_template = "delivery_consumer_log_es_bulk_key_{}"
    consumer_log_es_bulk_key_list = []
    for index in range(ConsumerLogConstants.CONSUMER_LOG_ES_BULK_THREAD_NUM):
        consumer_log_es_bulk_key_list.append(consumer_log_es_bulk_key_template.format(index))

    # 生成锁key 的值
    consumer_log_es_bulk_key_value_list = redis_store.mget(consumer_log_es_bulk_key_list)

    # 进行判断是否有可用锁
    consumer_log_es_bulk_key = None
    for index, value in enumerate(consumer_log_es_bulk_key_value_list):
        if not value:
            consumer_log_es_bulk_key = consumer_log_es_bulk_key_template.format(index)
            break

    if not consumer_log_es_bulk_key:
        return None

    # 进行锁20s
    redis_store.set(consumer_log_es_bulk_key, 1, ex=20)
    return consumer_log_es_bulk_key


class SqsPubLogDTO:
    msg_id = None
    address = None
    url = None
    k = None
    delay_seconds = None
    body = None
    group_id = None
    work_wait_time = None
    send_execute_time = None
    send_time = None
    send_success_time = None
    create_time = None


class SqsSubLogDTO:
    msg_id = None
    address = None
    url = None
    k = None
    body = None
    delay_seconds = None
    status = None
    receive_time = None
    receive_success_time = None
    message_wait_time = None
    work_wait_time = None
    execute_time = None
    create_time = None


def get_log_list(log_list_key, log_app_name):
    with RedisLock(
        lock_name="{}_consumer_log_es_{}_bulk_num_cat".format(log_app_name, log_list_key),
        timeout_ms=6000,
    ):
        # 获取现在的日志长度
        sub_log_len = redis_store.llen(log_list_key)
        if sub_log_len < ConsumerLogConstants.CONSUMER_LOG_ES_BULK_NUM:
            current_app.logger.info("数量不够")
            return [], ""

        # 获取锁key
        consumer_log_es_bulk_key = get_consumer_log_es_bulk_key()
        if not consumer_log_es_bulk_key:
            current_app.logger.info("没有获取到锁")
            return [], ""

        # 获取保存的日志列表
        pipe = redis_store.pipeline()
        for _ in range(ConsumerLogConstants.CONSUMER_LOG_ES_BULK_NUM):
            pipe.rpop(log_list_key)
        return pipe.execute(), consumer_log_es_bulk_key


class LogSqs:
    buffer_size = 1500
    buffer_cut = 500

    @staticmethod
    def save_producer_log(log_app_name, save_func):
        if not isfunction(save_func):
            return False
        # 获取需要保存的日志列表和锁key
        exec_results_start_time = time.time()
        results, consumer_log_es_bulk_key = get_log_list(
            ConsumerLogConstants.PUB_LOG.format(log_app_name), log_app_name
        )
        exec_results_end_time = time.time()
        if exec_results_end_time - exec_results_start_time > 1:
            current_app.logger.error(
                "producer_log: get_results:{}".format(
                    exec_results_end_time - exec_results_start_time
                )
            )

        if not results:
            return False

        batch_saves_start_time = time.time()
        try:
            batch_saves = []
            for m in results:
                if not m:
                    continue

                msg_ret = json.loads(m)
                msg_body = msg_ret["msg"]
                if isinstance(msg_body, dict):
                    try:
                        msg_body = json.dumps(msg_body)
                        msg_body = msg_body[0:2000]
                    except Exception:
                        current_app.logger.error(
                            "producer_log resolve msg body error:{}, "
                            "body:{}".format(traceback.format_exc(), msg_ret["body"])
                        )

                send_execute_time = -1
                try:
                    send_execute_time_seconds = (
                        datetime.strptime(msg_ret["send_success_time"], "%Y.%m.%d %H:%M:%S.%f")
                        - datetime.strptime(msg_ret["send_time"], "%Y.%m.%d %H:%M:%S.%f")
                    ).total_seconds()
                    send_execute_time = int(send_execute_time_seconds * 1000)
                except Exception:
                    pass

                work_wait_time = -1
                if "work_wait_time" in msg_ret:
                    work_wait_time = msg_ret["work_wait_time"]

                create_time = utils.format_timestamp(
                    utils.get_utc_timestamp(), formats="%Y.%m.%d %H:%M:%S.%f"
                )[:-3]
                send_time = msg_ret["send_time"]
                send_success_time = msg_ret["send_success_time"]

                pub_log = SqsPubLogDTO()
                pub_log.msg_id = msg_ret["msg_id"]
                pub_log.address = msg_ret["address"]
                pub_log.url = msg_ret["url"]
                pub_log.k = msg_ret["key"]
                pub_log.delay_seconds = msg_ret["delay_seconds"]
                pub_log.body = msg_body
                pub_log.group_id = msg_ret["group_id"]
                pub_log.work_wait_time = work_wait_time
                pub_log.send_execute_time = send_execute_time
                pub_log.send_time = send_time
                pub_log.send_success_time = send_success_time
                pub_log.create_time = create_time

                batch_saves.append(pub_log)

            if batch_saves:
                save_func(batch_saves)
        except Exception as e:
            current_app.logger.error(
                "producer_log batch_saves error:{}".format(traceback.format_exc())
            )
            raise e
        finally:
            # current_app.logger.error('producer_log batch_saves3')
            if consumer_log_es_bulk_key:
                redis_store.delete(consumer_log_es_bulk_key)

            batch_saves_total_time = time.time() - batch_saves_start_time
            if batch_saves_total_time > 15:
                current_app.logger.error(
                    "producer_log batch_saves_time:{}".format(batch_saves_total_time)
                )

        return True

    @staticmethod
    def save_consumer_log(log_app_name, save_func):
        if not isfunction(save_func):
            return False
        # 获取需要保存的日志列表和锁key
        exec_results_start_time = time.time()
        results, consumer_log_es_bulk_key = get_log_list(
            ConsumerLogConstants.SUB_LOG.format(log_app_name), log_app_name
        )
        exec_results_end_time = time.time()
        if exec_results_end_time - exec_results_start_time > 1:
            current_app.logger.error(
                "consumer_log: get_results:{}".format(
                    exec_results_end_time - exec_results_start_time
                )
            )

        if not results:
            return False

        batch_saves_start_time = time.time()
        try:
            batch_saves = []
            for m in results:
                if not m:
                    continue

                msg_ret = json.loads(m)
                msg_ret["create_time"] = utils.format_timestamp(
                    utils.get_utc_timestamp(), formats="%Y.%m.%d %H:%M:%S.%f"
                )[:-3]

                if "work_wait_time" not in msg_ret:
                    msg_ret["work_wait_time"] = -1
                if "message_wait_time" not in msg_ret:
                    msg_ret["message_wait_time"] = -1
                if "delay_seconds" not in msg_ret:
                    msg_ret["delay_seconds"] = -1

                msg_body = msg_ret["body"]
                if utils.is_not_empty(msg_body):
                    try:
                        msg_body_json = json.loads(msg_body)
                        if isinstance(msg_body_json, dict) and "t" in msg_body_json:
                            msg_body_msg = msg_body_json["msg"]
                            if utils.is_not_empty(msg_body_msg):
                                msg_body = json.dumps(msg_body_msg)
                            else:
                                msg_body = ""

                        msg_ret["body"] = msg_body[0:2000]
                    except Exception:
                        current_app.logger.error(
                            "consumer_log resolve msg body error:{}, "
                            "body:{}".format(traceback.format_exc(), msg_ret["body"])
                        )
                msg_ret["execute_time"] = -1
                try:
                    if "receive_success_time" in msg_ret and msg_ret["receive_success_time"]:
                        execute_time = (
                            datetime.strptime(
                                msg_ret["receive_success_time"], "%Y.%m.%d %H:%M:%S.%f"
                            )
                            - datetime.strptime(msg_ret["receive_time"], "%Y.%m.%d %H:%M:%S.%f")
                        ).total_seconds()
                        msg_ret["execute_time"] = int(execute_time * 1000)
                    else:
                        msg_ret["receive_success_time"] = msg_ret["receive_time"]
                except Exception:
                    pass

                sub_log = SqsSubLogDTO()
                sub_log.msg_id = msg_ret["msg_id"]
                sub_log.address = msg_ret["address"]
                sub_log.url = msg_ret["url"]
                sub_log.k = msg_ret["key"]
                sub_log.body = msg_ret["body"]
                sub_log.delay_seconds = msg_ret["delay_seconds"]
                sub_log.status = msg_ret["status"]
                sub_log.receive_time = msg_ret["receive_time"]
                sub_log.receive_success_time = msg_ret["receive_success_time"]
                sub_log.message_wait_time = msg_ret["message_wait_time"]
                sub_log.work_wait_time = msg_ret["work_wait_time"]
                sub_log.execute_time = msg_ret["execute_time"]
                sub_log.create_time = msg_ret["create_time"]

                batch_saves.append(sub_log)

            if batch_saves:
                save_func(batch_saves)

        except Exception as e:
            current_app.logger.error(
                "consumer_log batch_saves error:{}".format(traceback.format_exc())
            )
            raise e
        finally:
            # current_app.logger.error('consumer_log batch_saves3')
            if consumer_log_es_bulk_key:
                redis_store.delete(consumer_log_es_bulk_key)

            batch_saves_total_time = time.time() - batch_saves_start_time
            if batch_saves_total_time > 15:
                current_app.logger.error(
                    "consumer_log batch_saves_time:{}".format(batch_saves_total_time)
                )

        return True



from easi_py_common.sqs.consumer import subscribe as sub


def subscribe(
    address,
    work_number=8,  # 协程数
    msg_lifetime=86400 * 3,  # 消息过期时间(xx秒内未消费直接丢弃)
    visibility_timeout_seconds=300,
):  # 消息超时时间，超过xx秒未执行完成，会走重试流程
    def wrapper(f):
        sub(
            address,
            f,
            work_number=work_number,
            msg_lifetime=msg_lifetime,
            visibility_timeout_seconds=visibility_timeout_seconds,
        )
        return f

    return wrapper



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['easi_py_common',
 'easi_py_common.client',
 'easi_py_common.core',
 'easi_py_common.id',
 'easi_py_common.jwt',
 'easi_py_common.s3',
 'easi_py_common.secretsmanager',
 'easi_py_common.sqs',
 'easi_py_common.util']

package_data = \
{'': ['*']}

install_requires = \
['boto3==1.16.19',
 'flask-redis==0.4.0',
 'flask-sqlalchemy==2.4.4',
 'flask-uploads==0.2.1',
 'flask-wtf==0.14.3',
 'flask==1.1.2',
 'flaskerk==0.6.3',
 'gevent==20.9.0',
 'itsdangerous==1.1.0',
 'orjson==3.4.6',
 'pip==21.0.1',
 'pydantic==1.7.2',
 'pymysql==0.10.1',
 'pytz==2020.4',
 'pyyaml==5.3.1',
 'requests==2.25.0',
 'sqlalchemy==1.3.24',
 'werkzeug==1.0.1']

setup_kwargs = {
    'name': 'easi-py-common',
    'version': '0.0.76',
    'description': '',
    'long_description': None,
    'author': 'wangziqing',
    'author_email': 'eininst@aliyun.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://poetry.eustace.io/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import json
import traceback
from datetime import datetime

import flask
import orjson
from gevent.pool import Pool

from easi_py_common.core.redis import redis_store
from easi_py_common.sqs import constants
from easi_py_common.sqs.constants import ConsumerLogConstants
from easi_py_common.sqs.producer_session import ProducerSession
from easi_py_common.sqs.sqs import SQSClient

pool = Pool(128)

try:
    from flask import _app_ctx_stack as stack
except ImportError:
    from flask import _request_ctx_stack as stack


class Producer:
    def __init__(self):
        self.started = False
        self.sqs_client = None
        self.app = None
        self.sign = ""
        self.log_app_name = ""

    def __format_msg(self, msg):
        if isinstance(msg, (str, int, bool, float)):
            return msg
        else:
            return orjson.dumps(msg)

    def start(self, app: flask.app.Flask, log_app_name, max_pool_connections=50, max_attempts=3):
        self.app = app
        self.log_app_name = log_app_name

        if self.started:
            return

        self.started = True

        if self.app and self.app.config["ENV"] != "prod":
            if "sign" in app.config["PUBSUB"]:
                self.sign = app.config["PUBSUB"]["sign"] + "_"

        if "region_name" in app.config["PUBSUB"]:
            try:
                aws_access_key_id = app.config["AWS_ACCESS_KEY_ID"]
                aws_secret_access_key = app.config["AWS_SECRET_ACCESS_KEY"]
                self.sqs_client = SQSClient(
                    app.config["PUBSUB"]["region_name"],
                    queue_name_prefix=self.sign,
                    aws_access_key_id=aws_access_key_id,
                    aws_secret_access_key=aws_secret_access_key,
                    max_pool_connections=max_pool_connections,
                    max_attempts=max_attempts,
                )
            except Exception:
                if self.app:
                    self.app.logger.error(traceback.format_exc())

        self.init_batch_mode(self.app)

    def start_by_sqs_client(self, sqs_client):
        self.sqs_client = sqs_client

    def publish(self, address, msg, key="", group_id="", delay_seconds=0, sign=None):
        pool.spawn(
            self.send,
            address,
            msg,
            key=key,
            group_id=group_id,
            delay_seconds=delay_seconds,
            sign=sign,
        )

    def send(self, address, msg, key="", group_id="", delay_seconds=0, sign=None):
        if self.sqs_client is None:
            if self.app:
                self.app.logger.error("sqs_client is None")
            return False
        if sign is None:
            address = self.sign + address
        else:
            address = sign + address
        return self.sqs_client.send_message(
            address, msg, key=key, group_id=group_id, delay_seconds=delay_seconds, cb=self.__log
        )

    def get_sqs_client(self):
        return self.sqs_client

    @property
    def session(self):
        # type: () -> ProducerSession
        ctx = stack.top
        if ctx is not None:
            if not hasattr(ctx, "producer_session"):
                ctx.producer_session = ProducerSession(
                    self.sqs_client, self.sign, app=self.app, cb=None
                )
            return ctx.producer_session

    def commit(self):
        self.session.commit()

    def init_batch_mode(self, app):
        if hasattr(app, "teardown_appcontext"):
            app.teardown_appcontext(self.batch_mode_teardown)
        else:
            app.teardown_request(self.batch_mode_teardown)

    def batch_mode_teardown(self, exception):
        ctx = stack.top
        if hasattr(ctx, "producer_session"):
            ctx.producer_session.commit()
            # if exception is None:
            #     ctx.producer_session.commit()
            # else:
            #     ctx.producer_session.flush()
        return exception

    def __log(self, msg_id, msg_body):
        send_time = self.__format_timestamp(msg_body["t"])
        send_success_time = self.__format_timestamp(msg_body["s_t"])
        msg_body["send_time"] = send_time
        msg_body["send_success_time"] = send_success_time
        msg_body["msg_id"] = msg_id

        redis_pub_log_key = ConsumerLogConstants.PUB_LOG.format(self.log_app_name)
        producer_log_len = redis_store.lpush(redis_pub_log_key, json.dumps(msg_body))
        if producer_log_len >= ConsumerLogConstants.CONSUMER_LOG_MAX_SLEEP_ES_BULK_BUFFER_SIZE:
            redis_store.rpop(redis_pub_log_key)
            return
        # 计算是否可以进行保存
        can_bulk_save = producer_log_len % ConsumerLogConstants.CONSUMER_LOG_ES_BULK_NUM == 0
        # 如果不可以保存，并且小于缓存最大值，则直接返回
        if (
            not can_bulk_save
            and producer_log_len < ConsumerLogConstants.CONSUMER_LOG_ES_BULK_BUFFER_SIZE
        ):
            return
        # 如果可以保存 或者 是200的倍数 则尝试进行保存
        if (
            can_bulk_save
            or producer_log_len % 200 == 0
            or producer_log_len >= ConsumerLogConstants.CONSUMER_LOG_MAX_SLEEP_ES_BULK_BUFFER_SIZE
        ):
            self.publish(constants.PUB_LOG_BULK_SAVE, "")
            return

    def __format_timestamp(self, timestamp, is_millisecond=True):
        if is_millisecond:
            timestamp = round(timestamp / 1000.0, 2)
        date_array = datetime.utcfromtimestamp(timestamp)
        return date_array.strftime("%Y.%m.%d %H:%M:%S.%f")[:-3]


producer = Producer()



import json
import sys
import traceback
from functools import wraps

from flask import current_app, jsonify, make_response
from pydantic import ValidationError
from sqlalchemy.exc import IntegrityError


class ServiceException(Exception):
    """服务异常
    主要针对于一些逻辑异常，就抛出相关的错误。
    比如当前用户不存在，名称不能为空
    """

    def __init__(self, code=400, msg="", data=None, *args, **kwargs):
        self.code = code
        self.message = msg
        self.data = data
        super(ServiceException, self).__init__(*args, **kwargs)

    def __repr__(self):
        return "{}<code:{}, message:{}>".format(self.__class__.__name__, self.code, self.message)

    def __unicode__(self):
        return "{}<code:{}, message:{}>".format(self.__class__.__name__, self.code, self.message)


class InvalidParmsException(ServiceException):
    def __init__(self):
        super(InvalidParmsException, self).__init__(code=20000, msg="Invalid parameter")


class DataNotFoundException(ServiceException):
    def __init__(self):
        super(DataNotFoundException, self).__init__(msg="data is not found")


class InvalidTokenException(ServiceException):
    def __init__(self):
        super(InvalidTokenException, self).__init__(code=20001, msg="invalid access token")


class TokenExpiredException(ServiceException):
    def __init__(self):
        super(TokenExpiredException, self).__init__(code=20000, msg="token expired")


class RestClientException(ServiceException):
    def __init__(self, code=400, msg="", *args, **kwargs):
        self.code = code
        self.message = msg
        super(RestClientException, self).__init__(code=code, msg=msg, *args, **kwargs)


def intercept_service_exception(f):
    @wraps(f)
    def wrapped(*args, **kwargs):
        try:
            ret = f(*args, **kwargs)
            return ret
        except ServiceException as e:
            if e.data:
                return jsonify(code=e.code, message=e.message, data=e.data)
            return jsonify(code=e.code, message=e.message)

        except ValidationError as v:

            try:
                traceback_exception_list = traceback.format_exception(*sys.exc_info())
                if (
                    traceback_exception_list
                    and len(traceback_exception_list) >= 3
                    and traceback_exception_list[-3]
                ):
                    current_app.logger.error(
                        "validation error: {}\n{}".format(
                            json.dumps(v.errors()), traceback_exception_list[-3].split("\n")[0]
                        )
                    )
            except Exception:
                pass

            return make_response(
                jsonify(
                    {"code": 100, "message": "validation_error", "validation error": v.errors()}
                ),
                200,
            )
        except IntegrityError:
            return jsonify(code=1100, message="重复操作，请稍后再试")

    return wrapped


from typing import List


class BaseFlaskPydanticException(Exception):
    """Base exc class for all exception from this library"""

    pass


class InvalidIterableOfModelsException(BaseFlaskPydanticException):
    """This exception is raised if there is a failure during serialization of
    response object with `response_many=True`"""

    pass


class JsonBodyParsingError(BaseFlaskPydanticException):
    """Exception for error occurring during parsing of request body"""

    pass


class ManyModelValidationError(BaseFlaskPydanticException):
    """This exception is raised if there is a failure during validation of many
    models in an iterable"""

    def __init__(self, errors: List[dict], *args):
        self._errors = errors
        super().__init__(*args)

    def errors(self):
        return self._errors
